@@ -10,9 +10,11 @@
 # the full text of the license.
 
 import locale
+from logging import getLogger
 import os
 import sys
 
+from getpass import getpass
 from io import BytesIO
 
 if hasattr(sys.version_info, "major") and sys.version_info.major >= 3:
@@ -31,9 +33,10 @@ else:
 
 import requests
 
-from bugzilla import __version__, log
-from bugzilla.bug import _Bug, _User
+from .apiversion import __version__
+from .bug import _Bug, _User
 
+log = getLogger(__name__)
 
 # Backwards compatibility
 Bug = _Bug
@@ -400,7 +403,6 @@ class BugzillaBase(object):
         self._cookiejar = None
         self._sslverify = bool(sslverify)
 
-        self.logged_in = False
         self.bug_autorefresh = True
 
         # Bugzilla object state info that users shouldn't mess with
@@ -641,7 +643,6 @@ class BugzillaBase(object):
 
         try:
             ret = self._login(self.user, self.password)
-            self.logged_in = True
             self.password = ''
             log.info("login successful for user=%s", self.user)
             return ret
@@ -649,6 +650,42 @@ class BugzillaBase(object):
             e = sys.exc_info()[1]
             raise BugzillaError("Login failed: %s" % str(e.faultString))
 
+    def interactive_login(self, user=None, password=None, force=False):
+        """
+        Helper method to handle login for this bugzilla instance.
+
+        If a 'user' is provided or 'force' is set to True; or no cookie/token
+        file exists, a username/password authentication is attempted requesting
+        any information that is not available from the user.
+
+        If a cookie/token file exists, the call to the instance login method is
+        skipped.
+        """
+        if not force and user is None:
+            auths = {
+                'cookies': self.cookiefile,
+                'token': self.tokenfile,
+            }
+            for (method, source) in auths.items():
+                if source and os.path.exists(source):
+                    log.info(
+                        'Using %s in %s for authentication', method, source)
+                    return
+        elif not force:
+            log.error('No authentication information provided for login')
+
+        log.info('Using username/password for authentication')
+
+        if not user:
+            sys.stdout.write('Bugzilla Username: ')
+            user = sys.stdin.readline().strip()
+        if not password:
+            password = getpass('Bugzilla Password: ')
+
+        log.info('Logging in... ')
+        self.login(user, password)
+        log.info('Authorization cookie received.')
+
     def logout(self):
         '''Log out of bugzilla. Drops server connection and user info, and
         destroys authentication cookies.'''
@@ -656,7 +693,32 @@ class BugzillaBase(object):
         self.disconnect()
         self.user = ''
         self.password = ''
-        self.logged_in = False
+
+    @property
+    def logged_in(self):
+        """
+        This is True if this instance is logged in else False.
+
+        We test if this session is authenticated by calling the User.get()
+        XMLRPC method with ids set. Logged-out users cannot pass the 'ids'
+        parameter and will result in a 505 error.
+
+        For Bugzilla 5 and later, a new method, User.valid_login is available
+        to test the validity of the token. However, this will require that the
+        username be cached along with the token in order to work effectively in
+        all scenarios and is not currently used. For more information, refer to
+        the following url.
+
+        http://bugzilla.readthedocs.org/en/latest/api/core/v1/user.html#valid-login
+        """
+        try:
+            self._proxy.User.get({'ids': []})
+            return True
+        except Fault:
+            e = sys.exc_info()[1]
+            if e.faultCode == 505:
+                return False
+            raise e
 
 
     #############################################
@@ -944,8 +1006,8 @@ class BugzillaBase(object):
         if self._supports_getbug_extra_fields:
             getbugdata["extra_fields"] = extra_fields
 
-        log.debug("Calling Bug.get_bugs with: %s", getbugdata)
-        r = self._proxy.Bug.get_bugs(getbugdata)
+        log.debug("Calling Bug.get with: %s", getbugdata)
+        r = self._proxy.Bug.get(getbugdata)
 
         if self.bz_ver_major >= 4:
             bugdict = dict([(b['id'], b) for b in r['bugs']])
@@ -1101,7 +1163,7 @@ class BugzillaBase(object):
         ]:
             if val is not None:
                 raise RuntimeError("'%s' search not supported by this "
-                                   "bugzilla" % key)
+                                   "bugzilla's XMLRPC interface" % key)
 
         query = {
             "product": self._listify(product),
@@ -1479,7 +1541,11 @@ class BugzillaBase(object):
 
         att_uri = self._attachment_uri(attachid)
 
-        response = requests.get(att_uri, cookies=self._cookiejar, stream=True)
+        defaults = self._transport.request_defaults.copy()
+        defaults["headers"] = defaults["headers"].copy()
+        del(defaults["headers"]["Content-Type"])
+
+        response = requests.get(att_uri, stream=True, **defaults)
 
         ret = BytesIO()
         for chunk in response.iter_content(chunk_size=1024):
@@ -1540,7 +1606,8 @@ class BugzillaBase(object):
         target_milestone=None,
         target_release=None,
         url=None,
-        sub_component=None):
+        sub_component=None,
+        alias=None):
 
         localdict = {}
         if blocks:
@@ -1566,7 +1633,8 @@ class BugzillaBase(object):
                 resolution=resolution, severity=severity, status=status,
                 target_milestone=target_milestone,
                 target_release=target_release, url=url,
-                assigned_to=assigned_to, sub_component=sub_component)
+                assigned_to=assigned_to, sub_component=sub_component,
+                alias=alias)
 
         ret.update(localdict)
         return ret
@@ -1660,7 +1728,7 @@ class BugzillaBase(object):
         return ret and ret[0]
 
     def getusers(self, userlist):
-        '''Return a list of Users from bugzilla.
+        '''Return a list of Users from .
 
         :userlist: List of usernames to lookup
         :returns: List of User records
