@@ -143,6 +143,7 @@ class RHPartnerTest(BaseTest):
         dependson = "427301"
         comment = "Test bug from python-bugzilla test suite"
         sub_component = "Command-line tools (RHEL6)"
+        alias = "pybz-%s" % datetime.datetime.today().strftime("%s")
         newout = tests.clicomm("bugzilla new "
             "--product 'Red Hat Enterprise Linux 6' --version 6.0 "
             "--component lvm2 --sub-component '%s' "
@@ -150,9 +151,10 @@ class RHPartnerTest(BaseTest):
             "--comment \"%s\" "
             "--url %s --severity Urgent --priority Low --os %s "
             "--arch ppc --cc %s --blocked %s --dependson %s "
+            "--alias %s "
             "--outputformat \"%%{bug_id}\"" %
             (sub_component, summary, comment, url,
-             osval, cc, blocked, dependson), bz)
+             osval, cc, blocked, dependson, alias), bz)
 
         self.assertTrue(len(newout.splitlines()) == 3)
 
@@ -168,13 +170,21 @@ class RHPartnerTest(BaseTest):
         self.assertTrue(all([e in bug.cc for e in cc.split(",")]))
         self.assertEquals(bug.longdescs[0]["text"], comment)
         self.assertEquals(bug.sub_components, {"lvm2": [sub_component]})
+        self.assertEquals(bug.alias, [alias])
 
         # Close the bug
-        tests.clicomm("bugzilla modify --close WONTFIX %s" % bugid,
-                      bz)
+
+        # RHBZ makes it difficult to provide consistent semantics for
+        # 'alias' update:
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1173114
+        # alias += "-closed"
+        tests.clicomm("bugzilla modify "
+            "--close WONTFIX %s " %
+            bugid, bz)
         bug.refresh()
         self.assertEquals(bug.status, "CLOSED")
         self.assertEquals(bug.resolution, "WONTFIX")
+        self.assertEquals(bug.alias, [alias])
 
 
     def test5ModifyStatus(self):
@@ -289,7 +299,7 @@ class RHPartnerTest(BaseTest):
 
         bug.refresh()
         self.assertEquals(bug.cc, [])
-        self.assertEquals(bug.assigned_to, "wwoods@redhat.com")
+        self.assertEquals(bug.assigned_to, "crobinso@redhat.com")
         self.assertEquals(bug.qa_contact, "extras-qa@fedoraproject.org")
 
 
@@ -449,6 +459,14 @@ class RHPartnerTest(BaseTest):
         self.assertEquals(bug.platform, "s390")
         self.assertEquals(bug.url, "http://example.com/fribby")
 
+        # modify --field
+        tests.clicomm(cmd + "--field cf_fixed_in=foo-bar-1.2.3 \
+                      --field=cf_release_notes=baz", bz)
+
+        bug.refresh()
+        self.assertEquals(bug.fixed_in, "foo-bar-1.2.3")
+        self.assertEquals(bug.cf_release_notes, "baz")
+
 
     def test8Attachments(self):
         tmpdir = "__test_attach_output"
@@ -612,7 +630,17 @@ class RHPartnerTest(BaseTest):
                             "--user foobar@example.com "
                             "--password foobar login" % self.url, None,
                             expectfail=True)
-        self.assertTrue("Logging in... Login failed:" in ret)
+        self.assertTrue("Login failed: " in ret)
+
+
+    def test10LoginState(self):
+        bz = self.bzclass(url=self.url, cookiefile=None, tokenfile=None)
+        self.assertFalse(bz.logged_in,
+            "Login state check failed for logged out user.")
+
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
+        self.assertTrue(bz.logged_in,
+            "Login state check failed for logged in user.")
 
 
     def test11UserUpdate(self):
@@ -744,3 +772,105 @@ class RHPartnerTest(BaseTest):
         bz.update_bugs(bug.id, bz.build_update(sub_component={}))
         bug.refresh()
         self.assertEqual(bug.sub_components, {})
+
+    def _deleteAllExistingExternalTrackers(self, bugid):
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
+        ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
+        if ids != []:
+            bz.remove_external_tracker(ids=ids)
+
+    def test14ExternalTrackersQuery(self):
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
+        bugid = 461686
+        ext_bug_id = 1234659
+
+        # Delete any existing external trackers to get to a known state
+        self._deleteAllExistingExternalTrackers(bugid)
+
+        # test adding tracker
+        kwargs = {
+            'ext_type_url': 'http://bugzilla.mozilla.org',
+            'ext_type_description': 'Mozilla Foundation',
+            'ext_status': 'Original Status',
+        }
+
+        # Add a tracker that we can query for
+        bz.add_external_tracker(bugid, ext_bug_id, **kwargs)
+
+        kwargs['ext_bz_bug_id'] = ext_bug_id
+
+        boolean_query = bz.build_external_tracker_boolean_query(
+            ext_bz_bug_id=ext_bug_id)
+        query_results = bz.query(bz.build_query(boolean_query=boolean_query))
+        if len(query_results) != 1:
+            raise AssertionError(
+                'external tracker query by bug id should return 1 result.')
+        for key, value in kwargs.items():
+            assert kwargs[key] == value
+
+        boolean_query = bz.build_external_tracker_boolean_query(
+            ext_type_description=kwargs['ext_type_description'])
+        query_results = bz.query(bz.build_query(boolean_query=boolean_query))
+        assert bugid in [qr.id for qr in query_results]
+
+        boolean_query = bz.build_external_tracker_boolean_query(
+            ext_status=kwargs['ext_status'])
+        query_results = bz.query(bz.build_query(boolean_query=boolean_query))
+        if len(query_results) != 1:
+            raise AssertionError(
+                'external tracker query by status should return 1 result.')
+        for key, value in kwargs.items():
+            assert kwargs[key] == value
+
+        boolean_query = bz.build_external_tracker_boolean_query(
+            ext_type_url=kwargs['ext_type_url'])
+        query_results = bz.query(bz.build_query(boolean_query=boolean_query))
+        assert bugid in [qr.id for qr in query_results]
+
+    def test14ExternalTrackersAddUpdateRemoveQuery(self):
+        bz = self.bzclass(url=self.url, cookiefile=cf, tokenfile=tf)
+        bugid = 461686
+        ext_bug_id = 380489
+
+        # Delete any existing external trackers to get to a known state
+        self._deleteAllExistingExternalTrackers(bugid)
+
+        # test adding tracker
+        kwargs = {
+            'ext_type_id': 6,
+            'ext_type_url': 'http://bugzilla.mozilla.org',
+            'ext_type_description': 'Mozilla Foundation',
+            'ext_status': 'Original Status',
+            'ext_description': 'the description',
+            'ext_priority': 'the priority'
+        }
+        bz.add_external_tracker(bugid, ext_bug_id, **kwargs)
+        added_bug = bz.getbug(bugid).external_bugs[0]
+        assert added_bug['type']['id'] == kwargs['ext_type_id']
+        assert added_bug['type']['url'] == kwargs['ext_type_url']
+        assert (added_bug['type']['description'] ==
+            kwargs['ext_type_description'])
+        assert added_bug['ext_status'] == kwargs['ext_status']
+        assert added_bug['ext_description'] == kwargs['ext_description']
+        assert added_bug['ext_priority'] == kwargs['ext_priority']
+
+        # test updating status, description, and priority by id
+        kwargs = {
+            'ids': bz.getbug(bugid).external_bugs[0]['id'],
+            'ext_status': 'New Status',
+            'ext_description': 'New Description',
+            'ext_priority': 'New Priority'
+        }
+        bz.update_external_tracker(**kwargs)
+        updated_bug = bz.getbug(bugid).external_bugs[0]
+        assert updated_bug['ext_bz_bug_id'] == str(ext_bug_id)
+        assert updated_bug['ext_status'] == kwargs['ext_status']
+        assert updated_bug['ext_description'] == kwargs['ext_description']
+        assert updated_bug['ext_priority'] == kwargs['ext_priority']
+
+        # test removing tracker
+        ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
+        assert len(ids) == 1
+        bz.remove_external_tracker(ids=ids)
+        ids = [bug['id'] for bug in bz.getbug(bugid).external_bugs]
+        assert len(ids) == 0
